from flask import *
import sqlite3
from dotenv import *
import os
import pandas as pd
import plotly.graph_objects as go
import re
import altair as alt



adeo_bp = Blueprint('adeo', __name__)

columns = []
adeoDelayQueryPart = "WHERE Status_Final NOT IN ('On Track', 'On Hold', 'Finished', 'Closure', 'Not Started') AND Delay_Percent > 0.0"
adeoDelayQueryPartFilterd = "AND Status_Final NOT IN ('On Track', 'On Hold', 'Finished', 'Closure', 'Not Started') AND Delay_Percent > 0.0"

# Set pandas options to display all rows and columns
pd.set_option('display.max_rows', None)  # None to display all rows
pd.set_option('display.max_columns', None)  # None to display all columns
pd.set_option('display.width', None)  # No limit on the width of the display
pd.set_option('display.max_colwidth', None)  # No limit on the column width

def getSummaryInitiativeTable():
    # ---------- TABLE GENERATION QUERIES ------------

    # Connecting with Database
    conn = sqlite3.connect(os.getenv('DB_PATH'))
    cursor = conn.cursor()

    # TOTAL DELAY BUDGET BY INITIATIVE (CONVERTED TO BILLION)
    t_dbi_query = """

        SELECT 
        Initiative, 
        ROUND(SUM(Budget_Total) / 1000000000, 2) AS TotalBudget
    FROM 
        project_delay 
    WHERE 
        Status_Final NOT IN ('On Track', 'On Hold', 'Finished', 'Closure', 'Not Started') 
        AND Delay_Percent > 0.0 
    GROUP BY 
        Initiative;

    """
    t_dbi_rows = cursor.execute(t_dbi_query).fetchall()
    # print(t_dbi_rows)

    # TOTAL PROJECTS BY INITIATVE
    t_pi_query = "SELECT Initiative, COUNT(*) AS TotalProjectsByInitiative FROM project_delay GROUP BY Initiative;"
    t_pi_rows = cursor.execute(t_pi_query).fetchall()
    # print(t_pi_rows)

    # TOTAL PROJECTS BY EXECUTER (TO INSERT IN HEADER)
    t_pe_query = "SELECT Executer, COUNT(*) AS TotalProjectsByExecuter FROM project_delay GROUP BY Executer;"
    t_pe_rows = cursor.execute(t_pe_query).fetchall()
    #print(t_pi_rows)

    # TOTAL DELAY_PROJECTS BY INITIATVE
    t_dpi_query = "SELECT Initiative, COUNT(*) AS DelayedProjects FROM project_delay WHERE Status_Final NOT IN ('On Track', 'On Hold', 'Finished', 'Closure', 'Not Started') AND Delay_Percent > 0.0 GROUP BY Initiative;"
    t_dpi_rows = cursor.execute(t_dpi_query).fetchall()
    # print(t_dpi_rows)

    # DELAY PROJECTS BY EXECUTER  FOR VARIOUS INITIATIVES
    t_edp_query = """

        SELECT 
        Initiative, 
        Executer, 
        COUNT(*) AS DelayedProjects
    FROM 
        project_delay
    WHERE 
        Status_Final NOT IN ('On Track', 'On Hold', 'Finished', 'Closure', 'Not Started') 
        AND Delay_Percent > 0.0
    GROUP BY 
        Initiative, Executer
    ORDER BY 
        Initiative, Executer;

    """
    t_edp_rows = cursor.execute(t_edp_query).fetchall()
    # print(t_edp_rows)

    df_total = pd.DataFrame(t_pi_rows, columns=['Initiative', 'Total Projects']) # TOTAL  PROJECTS BY INITIATIVE
    df_delayed = pd.DataFrame(t_dpi_rows, columns=['Initiative', 'Total Delayed Projects']) # TOTAL  DELAY PROJECTS BY INITIATIVE
    df_total_executor = pd.DataFrame(t_pe_rows, columns=['Executer', 'Total Delayed Projects']) # TOTAL  PROJECTS BY EXECUTER
    df_total_delay_budget_initiative = pd.DataFrame(t_dbi_rows, columns=['Initiative', 'Total Budget (AED)']) # TOTAL Delay Budget BY EXECUTER
    # print(df_total_delay_budget_initiative)
    
    # DELAY PROJECTS BY EXECUTER  FOR VARIOUS INITIATIVES 
    df_exec = pd.DataFrame(t_edp_rows, columns=['Initiative', 'Executer', 'DelayedProjects'])
    # Pivot the DataFrame
    df_pivot = df_exec.pivot_table(
        index='Initiative',        # Rows are the initiatives
        columns='Executer',        # Columns are the executers
        values='DelayedProjects',  # The values to be filled in the table
        aggfunc='sum',             # Aggregate function in case of duplicate entries
        fill_value=0               # Fill NaN values with 0 (if an executer has no delays for an initiative)
    )

    # Reset the index for a cleaner output
    df_pivot.reset_index(inplace=True)

    # Code to add total projects by executer into column names of pivoted table
    executer_to_total = dict(zip(df_total_executor["Executer"], df_total_executor["Total Delayed Projects"]))
    # Update the columns of df_pivot
    updated_columns = [f"{col} ({executer_to_total[col]})" if col in executer_to_total else col for col in df_pivot.columns]
    df_pivot.columns = updated_columns

    # Display the pivoted DataFrame
    # print(df_pivot)

    
    # Merge df_total, df_delayed, and df_pivot in a single merge statement
    df_merge_final = pd.merge(pd.merge(pd.merge(df_total, df_delayed, on='Initiative', how='left'), df_total_delay_budget_initiative, on='Initiative', how='left'),df_pivot, on='Initiative', how='left')
    df_merge_final = df_merge_final.fillna(0)

    table_data = df_merge_final.to_dict(orient="records")
    table_columns = df_merge_final.columns

    # Display the final merged DataFrame
    # print(df_merge_final)

    conn.close()

    return table_data , table_columns , df_merge_final

def getSummaryExecuterTable():
    # ---------- TABLE GENERATION QUERIES ------------

    # Connecting with Database
    conn = sqlite3.connect(os.getenv('DB_PATH'))
    cursor = conn.cursor()

    # TOTAL DELAY BUDGET BY EXECUTER (CONVERTED TO BILLION)
    t_dbi_query = """

        SELECT 
        Executer, 
        ROUND(SUM(Budget_Total) / 1000000000, 2) AS TotalBudget
    FROM 
        project_delay 
    WHERE 
        Status_Final NOT IN ('On Track', 'On Hold', 'Finished', 'Closure', 'Not Started') 
        AND Delay_Percent > 0.0 
    GROUP BY 
        Executer;

    """
    t_dbi_rows = cursor.execute(t_dbi_query).fetchall()
    # print(t_dbi_rows)

    # TOTAL PROJECTS BY Executer
    t_pi_query = "SELECT Executer, COUNT(*) AS TotalProjectsByExecuter FROM project_delay GROUP BY Executer;"
    t_pi_rows = cursor.execute(t_pi_query).fetchall()
    # print(t_pi_rows)

    # TOTAL PROJECTS BY Initiative (TO INSERT IN HEADER)
    t_pe_query = "SELECT Initiative, COUNT(*) AS TotalProjectsByInitiative FROM project_delay GROUP BY Initiative;"
    t_pe_rows = cursor.execute(t_pe_query).fetchall()
    #print(t_pi_rows)

    # TOTAL DELAY_PROJECTS BY Executer
    t_dpi_query = "SELECT Executer, COUNT(*) AS DelayedProjects FROM project_delay WHERE Status_Final NOT IN ('On Track', 'On Hold', 'Finished', 'Closure', 'Not Started') AND Delay_Percent > 0.0 GROUP BY Executer;"
    t_dpi_rows = cursor.execute(t_dpi_query).fetchall()
    # print(t_dpi_rows)

    # DELAY PROJECTS BY Initiative  FOR VARIOUS Executers
    t_edp_query = """
        SELECT 
            Executer,
            Initiative,
            COUNT(*) AS DelayedProjects
        FROM 
            project_delay
        WHERE 
            Status_Final NOT IN ('On Track', 'On Hold', 'Finished', 'Closure', 'Not Started') 
            AND Delay_Percent > 0.0
        GROUP BY 
            Executer, Initiative
        ORDER BY 
            Executer, Initiative;
    """
    t_edp_rows = cursor.execute(t_edp_query).fetchall()
    # print(t_edp_rows)

    df_total = pd.DataFrame(t_pi_rows, columns=['Executer', 'Total Projects']) # TOTAL  PROJECTS BY EXECUTER
    df_delayed = pd.DataFrame(t_dpi_rows, columns=['Executer', 'Total Delayed Projects']) # TOTAL  DELAY PROJECTS BY EXECUTER
    df_total_executor = pd.DataFrame(t_pe_rows, columns=['Initiative', 'Total Delayed Projects']) # TOTAL  PROJECTS BY INITIATIVE
    df_total_delay_budget_initiative = pd.DataFrame(t_dbi_rows, columns=['Executer', 'Total Budget (AED)']) # TOTAL Delay Budget BY EXECUTER
    # print(df_total_delay_budget_initiative)

    df_total = df_total.fillna(0)
    df_delayed = df_delayed.fillna(0)
    df_total_executor = df_total_executor.fillna(0)
    df_total_delay_budget_initiative = df_total_delay_budget_initiative .fillna(0)

    
    # DELAY PROJECTS BY Initiative  FOR VARIOUS Executers 
    df_exec = pd.DataFrame(t_edp_rows, columns=['Executer', 'Initiative', 'DelayedProjects'])
    # Pivot the DataFrame
    df_pivot = df_exec.pivot_table(
        index='Executer',        # Rows are the initiatives
        columns='Initiative',        # Columns are the executers
        values='DelayedProjects',  # The values to be filled in the table
        aggfunc='sum',             # Aggregate function in case of duplicate entries
        fill_value=0               # Fill NaN values with 0 (if an executer has no delays for an initiative)
    )

    # Reset the index for a cleaner output
    df_pivot.reset_index(inplace=True)

    # Code to add total projects by executer into column names of pivoted table
    executer_to_total = dict(zip(df_total_executor["Initiative"], df_total_executor["Total Delayed Projects"]))
    # Update the columns of df_pivot
    updated_columns = [f"{col} ({executer_to_total[col]})" if col in executer_to_total else col for col in df_pivot.columns]
    df_pivot.columns = updated_columns

    # Display the pivoted DataFrame
    df_pivot = df_pivot.fillna(0)
    # print(df_pivot)

    
    # Merge df_total, df_delayed, and df_pivot in a single merge statement
    df_merge_final = pd.merge(pd.merge(pd.merge(df_total, df_delayed, on='Executer', how='left'), df_total_delay_budget_initiative, on='Executer', how='left'),df_pivot, on='Executer', how='left')
    df_merge_final = df_merge_final.fillna(0)

    table_data = df_merge_final.to_dict(orient="records")
    table_columns = df_merge_final.columns

    # Display the final merged DataFrame
    print(df_merge_final)
    # print(table_data)
    # print(table_columns)

    conn.close()

    return table_data , table_columns, df_merge_final

def generateSummaryInitiativeChart(df):
    # Extract initiatives and executors
    initiatives = df['Initiative']
    executors = df.columns[4:]  # Executors start from column 4 onward
    
    # Create the figure
    fig = go.Figure()
    
    # Initialize a list to track the cumulative height of each bar
    cumulative_heights = [0] * len(initiatives)
    
    # Add each executor's contribution as a stacked bar
    for executor in executors:
        # Clean the executor name by removing brackets and their contents
        clean_executor = re.sub(r"\s*\(.*?\)", "", executor)
        fig.add_trace(
            go.Bar(
                x=initiatives,
                y=df[executor],
                name=executor,  # Original name in the legend
                text=[clean_executor if val > 0 else '' for val in df[executor]],  # Cleaned name inside stack
                textposition="inside"  # Place the label inside the bar
            )
        )
        # Update cumulative heights for annotations
        cumulative_heights = [sum(x) for x in zip(cumulative_heights, df[executor])]

    # Add total delayed projects as annotations above each bar
    for i, initiative in enumerate(initiatives):
        fig.add_annotation(
            x=initiative,
            y=cumulative_heights[i],
            text=str(df['Total Delayed Projects'][i]),
            showarrow=False,
            font=dict(size=12, color="black"),
            yshift=10  # Shift the annotation slightly above the bar
        )

    # Wrap x-axis labels
    wrapped_labels = [
        '<br>'.join(label.split())  # Add line breaks to split long labels
        for label in initiatives
    ]

    # Update layout to match the example chart style
    fig.update_layout(
        barmode='stack',
        title=dict(
        text='Count of Delayed Projects By Initiative (Stacked By Executor)',
        x=0.45,  # Center the title
        y=0.97,  # Place the title closer to the top
        xanchor='center',  # Horizontal alignment
        yanchor='top',  # Vertical alignment
        font=dict(size=16)  # Title font size
         ),
        xaxis=dict(
            title=dict(
                text='Initiative',
                
            ),
            tickangle=0,  # Keep ticks horizontal
            tickvals=initiatives,
            ticktext=wrapped_labels,  # Use wrapped labels
        ),
        yaxis_title='Count of Delayed Projects',
        legend_title='Executor',
        template='plotly',
        height=350,
        autosize=True,
        margin=dict(l=0, r=0, t=40, b=5)  # Extremely tight margins
    )
    
    # Convert the plotly figure to HTML
    graph_html = fig.to_html(full_html=False)
    
    return graph_html


def generateSummaryExecuterChart(df):
    # Extract initiatives and executors
    executors = df['Executer']
    initiatives = df.columns[4:]  # Executors start from column 4 onward
    
    # Create the figure
    fig = go.Figure()
    
    # Initialize a list to track the cumulative height of each bar
    cumulative_heights = [0] * len(executors)
    
    # Add each executor's contribution as a stacked bar
    for initiative in initiatives:
        # Clean the executor name by removing brackets and their contents
        clean_initiative = re.sub(r"\s*\(.*?\)", "", initiative)
        fig.add_trace(
            go.Bar(
                x=executors,
                y=df[initiative],
                name=initiative,  # Original name in the legend
                text=[clean_initiative if val > 0 else '' for val in df[initiative]],  # Cleaned name inside stack
                textposition="inside"  # Place the label inside the bar
            )
        )
        # Update cumulative heights for annotations
        cumulative_heights = [sum(x) for x in zip(cumulative_heights, df[initiative])]

    # Add total delayed projects as annotations above each bar
    for i, executor in enumerate(executors):
        fig.add_annotation(
            x=executor,
            y=cumulative_heights[i],
            text=str(df['Total Delayed Projects'][i]),
            showarrow=False,
            font=dict(size=12, color="black"),
            yshift=10  # Shift the annotation slightly above the bar
        )
    
    # Update layout to match the example chart style
    fig.update_layout(
    barmode='stack',
    title=dict(
        text='Count of Delayed Projects By Executer (Stacked By Initiative)',
        x=0.45,  # Center the title
        y=0.97,  # Place the title closer to the top
        xanchor='center',  # Horizontal alignment
        yanchor='top',  # Vertical alignment
        font=dict(size=16)  # Title font size
    ),
    
    xaxis_title='Executer',
    yaxis_title='Count of Delayed Projects',
    legend_title='Initiative',
    template='plotly',
    autosize=True,
    height=350,
    # Reduce margins to minimum
    margin=dict(l=5, r=5, t=40, b=5)  # Extremely tight margins
    )

    
    # Convert the plotly figure to HTML
    graph_e_html = fig.to_html(full_html=False)
    
    return graph_e_html

def generateIndividualInitiativeChart(initiative):
    # Connecting with Database
    conn = sqlite3.connect(os.getenv('DB_PATH'))
    cursor = conn.cursor()

    # Modified query with parameterized initiative
    query = """
    SELECT SN, Executer, SDI, Delay_Percent 
    FROM project_delay 
    WHERE Status_Final NOT IN ('On Track', 'On Hold', 'Finished', 'Closure', 'Not Started') 
    AND Delay_Percent > 0.0 
    AND Initiative = ?;
    """

    # Execute the query with initiative as parameter
    cursor.execute(query, (initiative,))

    # Fetch results
    results = cursor.fetchall()
    # Column names
    columns = ['SN', 'Executer', 'SDI', 'Delay_Percent']

    # Create DataFrame
    df = pd.DataFrame(results, columns=columns)

    # Remove everything inside parentheses (and the parentheses themselves) from the 'Executer' column
    df['Executer'] = df['Executer'].apply(lambda x: re.sub(r'\(.*?\)', '', x).strip())

    # Filter out negative values from Delay_Percent
    df = df[df['Delay_Percent'] >= 0]

    # Create bar chart using Plotly
    fig = go.Figure()

    # Add bars for Delay_Percent (Color: Blue)
    fig.add_trace(go.Bar(
        x=df['Executer'] + " (" + df['SN'].astype(str) + ")",
        y=df['Delay_Percent'],
        text=df['Delay_Percent'],
        textposition='outside',
        marker_color='red',
        name="Delay Percentage",
        hovertemplate=df['Executer'] + " (" + df['SN'].astype(str) + ")<br>" + 
                      "Delay Percent: %{y}<extra></extra>",  # Hover shows Executer + SN and Delay Percent
        textfont=dict(size=12),  # Set a constant font size for the text labels
        textangle=90  # Rotate the text labels by 90 degrees
    ))

    # Add bars for SDI (Color: Red with hatched pattern)
    fig.add_trace(go.Bar(
        x=df['Executer'] + " (" + df['SN'].astype(str) + ")",
        y=df['SDI'],
        text=df['SDI'],
        textposition='outside',
        marker_color='lightgrey',  # Set color for SDI bars to red
        name="SDI",
        marker_pattern=dict(shape=""),  # Add hatched pattern ('x') to SDI bars
        hovertemplate=df['Executer'] + " (" + df['SN'].astype(str) + ")<br>" + 
                      "SDI: %{y}<extra></extra>",  # Hover shows Executer + SN and SDI value
        textfont=dict(size=12),  # Set a constant font size for the text labels
        textangle=90  # Rotate the text labels by 90 degrees
    ))

    # Add a dotted red line at 10% delay
    fig.add_trace(go.Scatter(
        x=df['Executer'] + " (" + df['SN'].astype(str) + ")",
        y=[10]*len(df),
        mode='lines',
        line=dict(color='red', dash='dot'),
        name='10% Delay'
    ))

    # Set y-axis to log scale
    fig.update_layout(
        yaxis_type="log",
        yaxis=dict(
            tickvals=[1, 2, 4, 6, 8, 10, 25, 50, 75, 100, 150, 200, 250, 300],
            ticktext=['1', '2', '4', '6', '8', '10', '25', '50', '75', '100', '150', '200', '250', '300'],
            range=[0, 2.4771],  # Log scale range corresponding to max value of 300
        ),
        title=f"Delay Percentage and SDI for Initiative: {initiative}",  # Dynamic title with initiative name
        xaxis_title="Executer (SN)",
        yaxis_title="Delay Percentage and SDI (log scale)",
        template="plotly_white",
        showlegend=True,
        xaxis_tickangle=-45,
        plot_bgcolor='white',
        paper_bgcolor='white',
        barmode='group',  # Group bars together
        height=500,  # Set the height of the chart
        autosize=True,
        bargap=0.2,  # Adjust the gap between bars (lower value for wider bars)
        bargroupgap=0,  # Adjust the gap between bar groups
    )

    # Show grid lines and styling
    fig.update_layout(
        xaxis=dict(showgrid=True, gridcolor="lightgrey", gridwidth=1, tickangle=90),  # Vertical x-axis labels
        yaxis=dict(showgrid=True, gridcolor="lightgrey", gridwidth=1)
    )

    # Add border to the plot area
    fig.add_shape(
        type="rect",
        x0=0, x1=1, y0=0, y1=1,
        xref="paper",
        yref="paper",
        line=dict(color="black", width=1)
    )

    # Adjust layout for title, legend, and margins
    fig.update_layout(
        margin=dict(l=5, r=5, t=40, b=5),
        title=dict(
            text=f"Delay Percentage and SDI for Initiative: {initiative}",
            x=0.205,
            y=0.97,
            xanchor='center',
            yanchor='top',
            font=dict(size=16)
        ),
        legend=dict(
            orientation="h",
            yanchor="bottom",
            y=1.02,
            xanchor="center",
            x=0.5
        )
    )

    # Return graph in HTML format
    graph_html_individual = '<a href="/adeo" class="flex items-left space-x-4 text-xs w-full justify-start pl-2 text-blue-600">Back to Summary Chart</a>'
    graph_html_individual += fig.to_html(full_html=False, div_id="graph_html")

    # Close the connection
    conn.close()

    return graph_html_individual


def generateIndividualExecuterChart(executer):
    # Connecting with Database
    conn = sqlite3.connect(os.getenv('DB_PATH'))
    cursor = conn.cursor()

    # Modified query with parameterized initiative
    query = """
    SELECT SN, Initiative, SDI, Delay_Percent 
    FROM project_delay 
    WHERE Status_Final NOT IN ('On Track', 'On Hold', 'Finished', 'Closure', 'Not Started') 
    AND Delay_Percent > 0.0 
    AND Executer = ?;
    """

    # Execute the query with initiative as parameter
    cursor.execute(query, (executer,))

    # Fetch results
    results = cursor.fetchall()
    # Column names
    columns = ['SN', 'Initiative', 'SDI', 'Delay_Percent']

    # Create DataFrame
    df = pd.DataFrame(results, columns=columns)

    # Remove everything inside parentheses (and the parentheses themselves) from the 'Initiative' column
    df['Initiative'] = df['Initiative'].apply(lambda x: re.sub(r'\(.*?\)', '', x).strip())

    # Filter out negative values from Delay_Percent
    df = df[df['Delay_Percent'] >= 0]

    # Create bar chart using Plotly
    fig = go.Figure()

    # Add bars for Delay_Percent
    fig.add_trace(go.Bar(
        x="(" + df['SN'].astype(str) + ")",  # Display only SN inside parentheses
        y=df['Delay_Percent'],
        text=df['Delay_Percent'],
        textposition='outside',
        marker_color='red',
        name="Delay Percentage",
        hovertemplate=df['Initiative'] + " (" + df['SN'].astype(str) + ")<br>" + 
                    "Delay Percent: %{y}<extra></extra>",  # Hover shows Initiative + SN and Delay Percent
        textfont=dict(size=12),  # Set a constant font size for the text labels
        textangle=90  # Rotate the text labels by 90 degrees
    ))

    # Add bars for SDI
    fig.add_trace(go.Bar(
        x="(" + df['SN'].astype(str) + ")",  # Display only SN inside parentheses
        y=df['SDI'],
        text=df['SDI'],
        textposition='outside',
        marker_color='lightgrey',
        name="SDI",
        marker_pattern=dict(shape=""),
        hovertemplate=df['Initiative'] + " (" + df['SN'].astype(str) + ")<br>" + 
                    "SDI: %{y}<extra></extra>",  # Hover shows Initiative + SN and SDI value
        textfont=dict(size=12),  # Set a constant font size for the text labels
        textangle=90  # Rotate the text labels by 90 degrees
    ))

    # Add a dotted red line at 10% delay
    fig.add_trace(go.Scatter(
        x = "(" + df['SN'].astype(str) + ")",
        y=[10]*len(df),
        mode='lines',
        line=dict(color='red', dash='dot'),
        name='10% Delay'
    ))

    # Set y-axis to log scale
    fig.update_layout(
        yaxis_type="log",
        yaxis=dict(
            tickvals=[1, 2, 4, 6, 8, 10, 25, 50, 75, 100, 150, 200, 250, 300],  # Log scale tick values
            ticktext=['1', '2', '4', '6', '8', '10', '25', '50', '75', '100', '150', '200', '250', '300'],  # Tick text
            range=[0, 2.4771],  # Log scale range corresponding to max value of 300
        ),
        title=f"Delay Percentage and SDI for Executer: {executer}",
        xaxis_title="(SN)",
        yaxis_title="Delay Percentage and SDI (log scale)",
        template="plotly_white",
        showlegend=True,
        plot_bgcolor='white',
        paper_bgcolor='white',
        barmode='group',  # Group bars together
        height=450,
        bargap=0.2,  # Remove gap between bars
        bargroupgap=0,  # Controls the gap between different categories (bars)
        autosize=True
    )


    # Add grid lines for both axes
    fig.update_layout(
        xaxis=dict(showgrid=True, gridcolor="lightgrey", gridwidth=1, tickangle=90),  # Vertical x-axis labels
        yaxis=dict(showgrid=True, gridcolor="lightgrey", gridwidth=1)
    )

    # Add border to the plot area
    fig.add_shape(
        type="rect",
        x0=0, x1=1, y0=0, y1=1,
        xref="paper",
        yref="paper",
        line=dict(color="black", width=1)
    )

    # Adjust layout for title, legend, and margins
    fig.update_layout(
        margin=dict(l=5, r=5, t=40, b=5),
        title=dict(
            text=f"Delay Percentage and SDI for Executer: {executer}",
            x=0.15,
            y=0.97,
            xanchor='center',
            yanchor='top',
            font=dict(size=16)
        ),
        legend=dict(
            orientation="h",
            yanchor="bottom",
            y=1.02,
            xanchor="center",
            x=0.5
        )
    )

    # Return graph in HTML format
    graph_html_individual = '<a href="/adeo" class="flex items-left space-x-4 text-xs w-full justify-start pl-2 ">Back to Summary Chart</a>'
    graph_html_individual += fig.to_html(full_html=False, div_id="graph_html_e")

    # Close the connection
    conn.close()

    return graph_html_individual

# -----------------------------------------------------------------
@adeo_bp.route("/adeo", methods=['GET','POST'])
def getAllRecords():
    columns = [] # Empty the columns to prevent double list

    # Connecting with Database
    conn = sqlite3.connect(os.getenv('DB_PATH'))
    cursor = conn.cursor()

    # Execute Query to get PRAGMA Info -> Column Info
    cursor.execute('PRAGMA table_info(project_delay);')
    pragma = cursor.fetchall()
    for i in pragma:
        columns.append(i[1])

    # Execute Query to get all Rows with adeo Delay
    init_query = 'SELECT * FROM project_delay'
    query = init_query + " " + adeoDelayQueryPart
    print(query)
    cursor.execute(query)
    rows = cursor.fetchall()
    conn.close()

    table_data , columnsTD, dfI = getSummaryInitiativeTable()
    table_data_e , columnsTD_e, dfE = getSummaryExecuterTable()

    # CHART LOGIC
    param_name = request.args.get('param_name')  # Extract GET parameter
    param_type = request.args.get('param_type')
    print(f"Received parameter: {param_name}")
    print(f"Received Type: {param_type}")


    if (param_name and param_type == "initiative"):
        graph_html = generateIndividualInitiativeChart(param_name)
        graph_html_e = ""
    elif (param_name and param_type == "executer"):
        graph_html = ""
        graph_html_e = generateIndividualExecuterChart(param_name)
    else:
        graph_html = generateSummaryInitiativeChart(dfI)
        graph_html_e = generateSummaryExecuterChart(dfE)


    # print(table_data_e)
    # print("\n\n\n")
    # print(columnsTD_e)
    # print(dfE)

    return render_template('adeo.html', rows = rows, columns=columns, table_data = table_data, columnsTD=columnsTD, table_data_e = table_data_e, columnsTD_e=columnsTD_e, graph_html=graph_html, graph_html_e=graph_html_e)


@adeo_bp.route("/adeo/filtered", methods=['POST'])
def getFilteredRecords():
    columns = []  # Empty the columns to prevent double list

    # Connecting with Database
    conn = sqlite3.connect(os.getenv('DB_PATH'))
    cursor = conn.cursor()

    # Execute Query to get PRAGMA Info -> Column Info
    cursor.execute('PRAGMA table_info(project_delay);')
    pragma = cursor.fetchall()
    for i in pragma:
        columns.append(i[1])

    # Get form data
    phases = request.form.getlist('phase')  # List of selected phases
    search_term = request.form.get('searchTerm')  # Search term from the form
    search_type = request.form.get('searchType', 'exact')  # Default to 'exact' if no type is selected
    selected_columns = request.form.getlist('columns')  # List of selected columns for search
    search_operator = request.form.get('operator')

    # Debugging print statements
    print(f"Selected Phases: {phases}")
    print(f"Search Term: {search_term}")
    print(f"Search Type: {search_type}")
    print(f"Selected Columns: {selected_columns}")
    print(f"Search Operator: {search_operator}")


    # Initialize query and parameters
    query = 'SELECT * FROM project_delay WHERE 1=1'  # Always true to add filters dynamically
    parameters = []

    # If no phase or "All Phases" is selected, do not filter by phase
    if phases and "All Phases" not in phases:  # Only apply phase filter if "All Phases" is NOT selected
        query += ' AND phase IN ({})'.format(','.join(['?'] * len(phases)))
        parameters.extend(phases)

    # Apply search logic based on search type and selected columns
    if search_term and selected_columns:
        search_term = f"%{search_term}%"  # Add wildcards for LIKE search
        conditions = []
        
        # Debugging print to show the column types and matching logic
        print(f"Checking column types and applying search operator for {len(selected_columns)} selected columns.")

        # Add conditions for selected columns based on column types
        for column in selected_columns:
            # Determine if column is of integer type (based on pragma)
            if column in columns:
                column_index = columns.index(column)
                column_type = pragma[column_index][2]  # column[2] is the type of the column
                
                if (column_type == 'INTEGER' or column_type == 'REAL'):
                    # For integer columns, we use exact match (`=`) 
                    if (search_operator == 'greater'):
                        search_operator = ">"
                    elif (search_operator == 'less'):
                        search_operator = "<"
                    else:
                        search_operator = "="  
                else:
                    # For text columns, apply LIKE or exact match based on search_type
                    search_operator = "LIKE" 

                # Debugging print for column type and operator applied
                print(f"Column: {column}, Type: {column_type}, Search Operator: {search_operator}")
                
                conditions.append(f"{column} {search_operator} ?")
                parameters.append(search_term if search_operator == "LIKE" else search_term.strip('%'))  # For exact match, strip wildcards

        # Add the constructed conditions to the query
        if conditions:
            query += ' AND (' + ' OR '.join(conditions) + ')'
            

    # Debugging print statements to show final query and parameters
    query += ' ' + adeoDelayQueryPartFilterd
    print("FINAL QUERY: ", query)
    print("FINAL PARAMETERS: ", parameters)

    # Execute query with parameters
    cursor.execute(query, parameters)
    rows = cursor.fetchall()

    conn.close()

    table_data , columnsTD, dfI = getSummaryInitiativeTable()
    table_data_e , columnsTD_e, dfE = getSummaryExecuterTable()

    graph_html = generateSummaryInitiativeChart(dfI)
    graph_html_e = generateSummaryExecuterChart(dfE)


    # print(table_data)
    return render_template('adeo.html', rows = rows, columns=columns, table_data = table_data, columnsTD=columnsTD, table_data_e = table_data_e, columnsTD_e=columnsTD_e, graph_html=graph_html, graph_html_e=graph_html_e)
